# -*- coding: utf-8 -*-
# Copyright (c) Polyconseil SAS. All rights reserved.

from __future__ import absolute_import
from __future__ import unicode_literals

import itertools
import logging

import docker.errors
import packaging.version

from . import __version__

logger = logging.getLogger(__name__)


def created_by_older_version(obj):
    grocker_version = packaging.version.parse(__version__)
    labels = obj.attrs.get('Config', obj.attrs)['Labels']
    obj_version = labels.get('grocker.version', '')
    return packaging.version.parse(obj_version) < grocker_version


def docker_purge_container(docker_client, current_version=False):
    """
    Purge Grocker internal containers.

    Args:
        docker_client (docker.DockerClient): a docker client
        current_version (bool): whether the images for current version will be deleted
    """
    removable_containers = [
        container
        for container in docker_client.containers.list(
            all=True,
            filters={'label': 'grocker.version', 'status': 'exited'},
        )
        if (
            (current_version or created_by_older_version(container))
            and container.attrs['Config']['Labels'].get('grocker.image.role') != 'runner'
        )
    ]

    for container in removable_containers:
        logger.info('Removing container %s...', container.name)
        try:
            container.remove()
        except docker.errors.APIError as e:
            logger.error(e)


def docker_purge_volumes(docker_client, current_version=False):
    """
    Purge Grocker volumes.

    Args:
        docker_client (docker.DockerClient): a docker client
        current_version (bool): whether the volumes for current version will be deleted
    """
    removable_volumes = [
        volume
        for volume in itertools.chain(
            docker_client.volumes.list(filters={'label': 'grocker.version'}),
            docker_client.volumes.list(filters={'label': 'grocker'}),  # old grocker versions
        )
        if current_version or created_by_older_version(volume)
    ]

    for volume in removable_volumes:
        logger.info('Removing volume %s...', volume.name)
        try:
            volume.remove()
        except docker.errors.APIError as e:
            logger.error(e)


def docker_purge_images(docker_client, current_version=False, runner=False):
    """
    Purge Grocker images.

    Args:
        docker_client (docker.DockerClient): a docker client
        current_version (bool): whether the images for current version will be deleted
        runner (bool): whether the runner images will be deleted
    """
    removable_images = [
        image
        for image in docker_client.images.list(filters={'label': 'grocker.version'})
        if (
            (current_version or created_by_older_version(image))
            and (runner or image.attrs.get('Labels', {}).get('grocker.image.role') == 'runner')
        )
    ]
    for image in removable_images:
        for tag in image.tags:
            logger.info('Removing image %s...', tag)
            try:
                docker_client.images.remove(tag)
            except docker.errors.APIError as e:
                logger.error(e)



# -*- coding: utf-8 -*-
# Copyright (c) Polyconseil SAS. All rights reserved.

from __future__ import absolute_import
from __future__ import unicode_literals

import logging
import logging.config


class ColoredStreamHandler(logging.StreamHandler):
    _colors = {
        logging.DEBUG: '36',
        logging.INFO: '1;36',
        logging.WARNING: '1;33',
        logging.ERROR: '1;31',
        logging.CRITICAL: '1;41;37',
    }

    def format(self, record):
        msg = super(ColoredStreamHandler, self).format(record)
        color = self._colors.get(record.levelno, None)

        if not color or not self.stream.isatty():
            return msg

        return '\x1b[{color}m{msg}\x1b[0m'.format(color=color, msg=msg)


def setup(verbose=False):
    level = 'DEBUG' if verbose else 'INFO'

    logging.config.dictConfig({
        'version': 1,
        'formatters': {
            'simple': {'format': '%(name)s: %(message)s'},
        },
        'handlers': {
            'console': {'class': '{}.ColoredStreamHandler'.format(__name__), 'formatter': 'simple'},
        },
        'loggers': {
            'grocker': {'handlers': ['console'], 'level': level},
        },
    })


if __name__ == '__main__':
    setup(verbose=True)
    test_logger = logging.getLogger(__package__)
    test_logger.debug('Some debug output.')
    test_logger.info('Some info output.')
    test_logger.warning('Some warning output.')
    test_logger.error('Some error output.')
    test_logger.critical('Some critical output.')



# -*- coding: utf-8 -*-
# Copyright (c) Polyconseil SAS. All rights reserved.
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import argparse
import base64
import logging
import logging.config
import os
import os.path
import subprocess
import tempfile
import zlib

try:  # Python 3+
    import configparser
except ImportError:  # Python 2.7
    import ConfigParser as configparser


WHEELS_DIRECTORY = os.path.expanduser('~/packages')


def arg_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('--python', default='python')
    parser.add_argument('--no-color', action='store_true')
    parser.add_argument('release', nargs='+')

    return parser


def setup_logging(enable_colors):
    colors = {'begin': '\x1b[1;36m', 'end': '\x1b[0m'} if enable_colors else {'begin': '', 'end': ''}

    logging.config.dictConfig({
        'version': 1,
        'formatters': {
            'simple': {'format': '{begin}%(message)s{end}'.format(**colors)},
        },
        'handlers': {
            'console': {'class': 'logging.StreamHandler', 'formatter': 'simple'},
        },
        'loggers': {
            __name__: {'handlers': ['console'], 'level': 'INFO'},
        },
    })


def info(*msg):
    logging.getLogger(__name__).info(*msg)


def setup_pip(venv, package_dir):
    """Generate venv pip.conf."""
    info('Setup pip...')

    # Standard config
    guest_config = configparser.ConfigParser()
    guest_config.add_section('global')
    guest_config.set('global', 'wheel-dir', package_dir)
    guest_config.set('global', 'find-links', package_dir)

    # Write config
    venv_pip_conf = os.path.join(venv, 'pip.conf')
    with open(venv_pip_conf, 'w') as f:
        guest_config.write(f)


def setup_venv(python):
    """Return python interpreter after setup the venv."""
    info('Setup venv using %s...', python)
    venv = tempfile.mkdtemp(suffix='.venv')
    try:
        # python 3
        subprocess.check_call([python, '-m', 'venv', venv])  # noqa: B603
    except subprocess.CalledProcessError:
        subprocess.check_call([python, '-m', 'virtualenv', venv])  # noqa: B603
    subprocess.check_call(  # noqa: B603
        [os.path.join(venv, 'bin', 'pip'), 'install', '-U', 'pip', 'setuptools', 'wheel'],
    )
    return venv


def build_wheels(venv, package, package_dir, constraint=None):
    info('Building wheels for %s...', package)
    pip = os.path.join(venv, 'bin', 'pip')
    constraint_args = ['--constraint', constraint] if constraint else []
    try:
        subprocess.check_call(  # noqa: B603
            [pip, 'wheel', '--wheel-dir', package_dir]
            + constraint_args
            + [package],
        )
        return True
    except subprocess.CalledProcessError as exc:
        info(str(exc))
        if exc.output:
            print(exc.output)
        return False


def main():
    parser = arg_parser()
    args = parser.parse_args()
    setup_logging(not args.no_color)

    venv = setup_venv(args.python)
    setup_pip(venv, WHEELS_DIRECTORY)

    constraints = os.environ.get('PIP_CONSTRAINT_CONTENT', base64.b64encode(zlib.compress(b'')))
    with tempfile.NamedTemporaryFile() as fp:
        fp.write(zlib.decompress(base64.b64decode(constraints)))
        fp.flush()

        for release in args.release:
            if not build_wheels(venv, release, WHEELS_DIRECTORY, fp.name):
                exit(1)


if __name__ == '__main__':
    main()



#! /usr/bin/env python
from setuptools import setup

setup(
    setup_requires=['setuptools>=30.3'],
    package_dir={  # FIXME: wait for https://github.com/pypa/setuptools/issues/1136
        '': 'src',
    },
)



# -*- coding: utf-8 -*-
# Copyright (c) Polyconseil SAS. All rights reserved.

from __future__ import absolute_import
from __future__ import unicode_literals

from . import build
from .build import build_runner_image
from . import naming
from . import op
from .op import docker_push_image, get_manifest_digest, is_prefixed_image
from .wheels import compile_wheels


__all__ = [
    'build_runner_image',
    'docker_push_image',
    'is_prefixed_image',
    'compile_wheels',
    'get_manifest_digest',
    'get_or_build_root_image',
    'get_or_build_compiler_image',
]


def get_or_build_root_image(docker_client, config):
    return op.docker_get_or_build_image(
        docker_client,
        naming.image_name(config, 'root'),
        lambda client: build.build_root_image(client, config),
    )


def get_or_build_compiler_image(docker_client, config):
    return op.docker_get_or_build_image(
        docker_client,
        naming.image_name(config, 'compiler'),
        lambda client: build.build_compiler_image(client, config),
    )
